//Written by: Holguer A Becerra
//ECE 31289 UPB-2014-I
//Semillero ADT UPB Bucaramanga
//Based on Xilinx examples

module timing_generator
#(
  parameter          VIDEO_MODE = "VGA"
  )
  (output reg active_q,
  output reg vsync, 
  output reg hsync,
  output reg VGA_HSYNC, 
  output reg VGA_VSYNC,
  output reg de,
  output   [11:0] bgnd_hcount,
  output   [11:0] bgnd_vcount,
  input pclk, input reset);

///////////////////////////////////////////////////////////////////////////
// Video Timing Parameters
///////////////////////////////////////////////////////////////////////////
  parameter SW_VGA       = 4'b0000;
  parameter SW_SVGA      = 4'b0001;
  parameter SW_XGA       = 4'b0011;
  parameter SW_WXGA       = 4'b1000;
  parameter SW_WXGA2       = 4'b1001;
  parameter SW_HDTV720P  = 4'b0010;
  parameter SW_HDTV1080P  = 4'b1110;
  parameter SW_HD  = 4'b1011;
  
  
  //1920x1080@25HZ 74.25MHz
  parameter HPIXELS_HDTV1080P = 11'd1920; //Horizontal Live Pixels
  parameter VLINES_HDTV1080P  = 11'd1080;  //Vertical Live ines
  parameter HSYNCPW_HDTV1080P = 11'd44;  //HSYNC Pulse Width
  parameter VSYNCPW_HDTV1080P = 11'd5;    //VSYNC Pulse Width
  parameter HFNPRCH_HDTV1080P = 11'd148;   //Horizontal Front Portch
  parameter VFNPRCH_HDTV1080P = 11'd36;    //Vertical Front Portch
  parameter HBKPRCH_HDTV1080P = 11'd528;  //Horizontal Front Portch
  parameter VBKPRCH_HDTV1080P = 11'd4;   //Vertical Front Portch
  

  //1360x768@60HZ 74.125MHz
  parameter HPIXELS_HD = 11'd1360; //Horizontal Live Pixels
  parameter VLINES_HD  = 11'd768;  //Vertical Live ines
  parameter HSYNCPW_HD = 11'd32;  //HSYNC Pulse Width
  parameter VSYNCPW_HD = 11'd5;    //VSYNC Pulse Width
  parameter HFNPRCH_HD = 11'd48;   //Horizontal Front Portch
  parameter VFNPRCH_HD = 11'd3;    //Vertical Front Portch
  parameter HBKPRCH_HD = 11'd80;  //Horizontal Front Portch
  parameter VBKPRCH_HD = 11'd37;   //Vertical Front Portch
  
 
  //1280x800@60HZ 71MHz
  parameter HPIXELS_WXGA2 = 11'd1280; //Horizontal Live Pixels
  parameter VLINES_WXGA2  = 11'd800;  //Vertical Live ines
  parameter HSYNCPW_WXGA2 = 11'd32;  //HSYNC Pulse Width
  parameter VSYNCPW_WXGA2 = 11'd6;    //VSYNC Pulse Width
  parameter HFNPRCH_WXGA2 = 11'd48;   //Horizontal Front Portch
  parameter VFNPRCH_WXGA2 = 11'd3;    //Vertical Front Portch
  parameter HBKPRCH_WXGA2 = 11'd80;  //Horizontal Front Portch
  parameter VBKPRCH_WXGA2 = 11'd14;   //Vertical Front Portch
  
  //1280x768@60HZ 68.250MHz
  parameter HPIXELS_WXGA = 11'd1280; //Horizontal Live Pixels
  parameter VLINES_WXGA  = 11'd768;  //Vertical Live ines
  parameter HSYNCPW_WXGA = 11'd32;  //HSYNC Pulse Width
  parameter VSYNCPW_WXGA = 11'd7;    //VSYNC Pulse Width
  parameter HFNPRCH_WXGA = 11'd48;   //Horizontal Front Portch
  parameter VFNPRCH_WXGA = 11'd3;    //Vertical Front Portch
  parameter HBKPRCH_WXGA = 11'd80;  //Horizontal Front Portch
  parameter VBKPRCH_WXGA = 11'd12;   //Vertical Front Portch

	//1280x720@60HZ
  parameter HPIXELS_HDTV720P = 11'd1280; //Horizontal Live Pixels
  parameter VLINES_HDTV720P  = 11'd720;  //Vertical Live ines
  parameter HSYNCPW_HDTV720P = 11'd80;  //HSYNC Pulse Width
  parameter VSYNCPW_HDTV720P = 11'd5;    //VSYNC Pulse Width
  parameter HFNPRCH_HDTV720P = 11'd72;   //Horizontal Front Portch
  parameter VFNPRCH_HDTV720P = 11'd3;    //Vertical Front Portch
  parameter HBKPRCH_HDTV720P = 11'd216;  //Horizontal Front Portch
  parameter VBKPRCH_HDTV720P = 11'd22;   //Vertical Front Portch
  
  
  //1024x768@60HZ  65MHz
  parameter HPIXELS_XGA = 11'd1024; //Horizontal Live Pixels
  parameter VLINES_XGA  = 11'd768;  //Vertical Live ines
  parameter HSYNCPW_XGA = 11'd136;  //HSYNC Pulse Width
  parameter VSYNCPW_XGA = 11'd6;    //VSYNC Pulse Width
  parameter HFNPRCH_XGA = 11'd24;   //Horizontal Front Portch
  parameter VFNPRCH_XGA = 11'd3;    //Vertical Front Portch
  parameter HBKPRCH_XGA = 11'd160;  //Horizontal Front Portch
  parameter VBKPRCH_XGA = 11'd29;   //Vertical Front Portch

  //800x600@60HZ 40MHz
  parameter HPIXELS_SVGA = 11'd800; //Horizontal Live Pixels
  parameter VLINES_SVGA  = 11'd600; //Vertical Live ines
  parameter HSYNCPW_SVGA = 11'd128; //HSYNC Pulse Width
  parameter VSYNCPW_SVGA = 11'd4;   //VSYNC Pulse Width
  parameter HFNPRCH_SVGA = 11'd40;  //Horizontal Front Portch
  parameter VFNPRCH_SVGA = 11'd1;   //Vertical Front Portch
  parameter HBKPRCH_SVGA = 11'd88;  //Horizontal Front Portch
  parameter VBKPRCH_SVGA = 11'd23;  //Vertical Front Portch
  
  //640x480@60HZ 25.152MHz
  parameter HPIXELS_VGA = 11'd640; //Horizontal Live Pixels
  parameter VLINES_VGA  = 11'd480; //Vertical Live ines
  parameter HSYNCPW_VGA = 11'd96;  //HSYNC Pulse Width
  parameter VSYNCPW_VGA = 11'd2;   //VSYNC Pulse Width
  parameter HFNPRCH_VGA = 11'd16;  //Horizontal Front Portch
  parameter VFNPRCH_VGA = 11'd11;  //Vertical Front Portch
  parameter HBKPRCH_VGA = 11'd48;  //Horizontal Front Portch
  parameter VBKPRCH_VGA = 11'd31;  //Vertical Front Portch
  

wire [3:0]SW;  
 
generate
    if (VIDEO_MODE == "VGA")
	 begin
			assign SW[3:0]=SW_VGA;
	 end
	 else if(VIDEO_MODE == "SVGA")
	 begin
			assign SW[3:0]=SW_SVGA;
	 end
	 else if(VIDEO_MODE == "XGA")
	 begin
			assign SW[3:0]=SW_XGA;
	 end
	 else if(VIDEO_MODE == "WXGA")
	 begin
			assign SW[3:0]=SW_WXGA;
	 end
	 else if(VIDEO_MODE == "WXGA2")
	 begin
			assign SW[3:0]=SW_WXGA2;
	 end
	 else if(VIDEO_MODE == "HDTV720P")
	 begin
			assign SW[3:0]=SW_HDTV720P;
	 end
	 else if(VIDEO_MODE == "HDTV1080P")
	 begin
			assign SW[3:0]=SW_HDTV1080P;
	 end
	 else if(VIDEO_MODE == "HD")
	 begin
			assign SW[3:0]=SW_HD;
	 end
endgenerate	 

	
  reg [11:0] tc_hsblnk;
  reg [11:0] tc_hssync;
  reg [11:0] tc_hesync;
  reg [11:0] tc_heblnk;
  reg [11:0] tc_vsblnk;
  reg [11:0] tc_vssync;
  reg [11:0] tc_vesync;
  reg [11:0] tc_veblnk;

 
  reg hvsync_polarity; //1-Negative, 0-Positive
  always @ (*)
  begin
    case (SW)
      SW_VGA:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_VGA - 11'd1;
        tc_hssync = HPIXELS_VGA - 11'd1 + HFNPRCH_VGA;
        tc_hesync = HPIXELS_VGA - 11'd1 + HFNPRCH_VGA + HSYNCPW_VGA;
        tc_heblnk = HPIXELS_VGA - 11'd1 + HFNPRCH_VGA + HSYNCPW_VGA + HBKPRCH_VGA;
        tc_vsblnk =  VLINES_VGA - 11'd1;
        tc_vssync =  VLINES_VGA - 11'd1 + VFNPRCH_VGA;
        tc_vesync =  VLINES_VGA - 11'd1 + VFNPRCH_VGA + VSYNCPW_VGA;
        tc_veblnk =  VLINES_VGA - 11'd1 + VFNPRCH_VGA + VSYNCPW_VGA + VBKPRCH_VGA;
      end

      SW_SVGA:
      begin
        hvsync_polarity = 1'b0;

        tc_hsblnk = HPIXELS_SVGA - 11'd1;
        tc_hssync = HPIXELS_SVGA - 11'd1 + HFNPRCH_SVGA;
        tc_hesync = HPIXELS_SVGA - 11'd1 + HFNPRCH_SVGA + HSYNCPW_SVGA;
        tc_heblnk = HPIXELS_SVGA - 11'd1 + HFNPRCH_SVGA + HSYNCPW_SVGA + HBKPRCH_SVGA;
        tc_vsblnk =  VLINES_SVGA - 11'd1;
        tc_vssync =  VLINES_SVGA - 11'd1 + VFNPRCH_SVGA;
        tc_vesync =  VLINES_SVGA - 11'd1 + VFNPRCH_SVGA + VSYNCPW_SVGA;
        tc_veblnk =  VLINES_SVGA - 11'd1 + VFNPRCH_SVGA + VSYNCPW_SVGA + VBKPRCH_SVGA;
      end

      SW_XGA:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_XGA - 11'd1;
        tc_hssync = HPIXELS_XGA - 11'd1 + HFNPRCH_XGA;
        tc_hesync = HPIXELS_XGA - 11'd1 + HFNPRCH_XGA + HSYNCPW_XGA;
        tc_heblnk = HPIXELS_XGA - 11'd1 + HFNPRCH_XGA + HSYNCPW_XGA + HBKPRCH_XGA;
        tc_vsblnk =  VLINES_XGA - 11'd1;
        tc_vssync =  VLINES_XGA - 11'd1 + VFNPRCH_XGA;
        tc_vesync =  VLINES_XGA - 11'd1 + VFNPRCH_XGA + VSYNCPW_XGA;
        tc_veblnk =  VLINES_XGA - 11'd1 + VFNPRCH_XGA + VSYNCPW_XGA + VBKPRCH_XGA;
      end
		SW_WXGA:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_WXGA - 11'd1;
        tc_hssync = HPIXELS_WXGA - 11'd1 + HFNPRCH_WXGA;
        tc_hesync = HPIXELS_WXGA - 11'd1 + HFNPRCH_WXGA + HSYNCPW_WXGA;
        tc_heblnk = HPIXELS_WXGA - 11'd1 + HFNPRCH_WXGA + HSYNCPW_WXGA + HBKPRCH_WXGA;
        tc_vsblnk =  VLINES_WXGA - 11'd1;
        tc_vssync =  VLINES_WXGA - 11'd1 + VFNPRCH_WXGA;
        tc_vesync =  VLINES_WXGA - 11'd1 + VFNPRCH_WXGA + VSYNCPW_WXGA;
        tc_veblnk =  VLINES_WXGA - 11'd1 + VFNPRCH_WXGA + VSYNCPW_WXGA + VBKPRCH_WXGA;
      end
		SW_WXGA2:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_WXGA2 - 11'd1;
        tc_hssync = HPIXELS_WXGA2 - 11'd1 + HFNPRCH_WXGA2;
        tc_hesync = HPIXELS_WXGA2 - 11'd1 + HFNPRCH_WXGA2 + HSYNCPW_WXGA2;
        tc_heblnk = HPIXELS_WXGA2 - 11'd1 + HFNPRCH_WXGA2 + HSYNCPW_WXGA2 + HBKPRCH_WXGA2;
        tc_vsblnk =  VLINES_WXGA2 - 11'd1;
        tc_vssync =  VLINES_WXGA2 - 11'd1 + VFNPRCH_WXGA2;
        tc_vesync =  VLINES_WXGA2 - 11'd1 + VFNPRCH_WXGA2 + VSYNCPW_WXGA2;
        tc_veblnk =  VLINES_WXGA2 - 11'd1 + VFNPRCH_WXGA2 + VSYNCPW_WXGA2 + VBKPRCH_WXGA2;
      end
		SW_HD:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_HD - 11'd1;
        tc_hssync = HPIXELS_HD - 11'd1 + HFNPRCH_HD;
        tc_hesync = HPIXELS_HD - 11'd1 + HFNPRCH_HD + HSYNCPW_HD;
        tc_heblnk = HPIXELS_HD - 11'd1 + HFNPRCH_HD + HSYNCPW_HD + HBKPRCH_HD;
        tc_vsblnk =  VLINES_HD - 11'd1;
        tc_vssync =  VLINES_HD - 11'd1 + VFNPRCH_HD;
        tc_vesync =  VLINES_HD - 11'd1 + VFNPRCH_HD + VSYNCPW_HD;
        tc_veblnk =  VLINES_HD - 11'd1 + VFNPRCH_HD + VSYNCPW_HD + VBKPRCH_HD;
      end
		SW_HDTV1080P:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_HDTV1080P - 11'd1;
        tc_hssync = HPIXELS_HDTV1080P - 11'd1 + HFNPRCH_HDTV1080P;
        tc_hesync = HPIXELS_HDTV1080P - 11'd1 + HFNPRCH_HDTV1080P + HSYNCPW_HDTV1080P;
        tc_heblnk = HPIXELS_HDTV1080P - 11'd1 + HFNPRCH_HDTV1080P + HSYNCPW_HDTV1080P + HBKPRCH_HDTV1080P;
        tc_vsblnk =  VLINES_HDTV1080P - 11'd1;
        tc_vssync =  VLINES_HDTV1080P - 11'd1 + VFNPRCH_HDTV1080P;
        tc_vesync =  VLINES_HDTV1080P - 11'd1 + VFNPRCH_HDTV1080P + VSYNCPW_HDTV1080P;
        tc_veblnk =  VLINES_HDTV1080P - 11'd1 + VFNPRCH_HDTV1080P + VSYNCPW_HDTV1080P + VBKPRCH_HDTV1080P;
      end
      default: //SW_HDTV720P:
      begin
        hvsync_polarity = 1'b1;

        tc_hsblnk = HPIXELS_HDTV720P - 11'd1;
        tc_hssync = HPIXELS_HDTV720P - 11'd1 + HFNPRCH_HDTV720P;
        tc_hesync = HPIXELS_HDTV720P - 11'd1 + HFNPRCH_HDTV720P + HSYNCPW_HDTV720P;
        tc_heblnk = HPIXELS_HDTV720P - 11'd1 + HFNPRCH_HDTV720P + HSYNCPW_HDTV720P + HBKPRCH_HDTV720P;
        tc_vsblnk =  VLINES_HDTV720P - 11'd1;
        tc_vssync =  VLINES_HDTV720P - 11'd1 + VFNPRCH_HDTV720P;
        tc_vesync =  VLINES_HDTV720P - 11'd1 + VFNPRCH_HDTV720P + VSYNCPW_HDTV720P;
        tc_veblnk =  VLINES_HDTV720P - 11'd1 + VFNPRCH_HDTV720P + VSYNCPW_HDTV720P + VBKPRCH_HDTV720P;
      end
    endcase
  end

  wire VGA_HSYNC_INT, VGA_VSYNC_INT;
  wire          bgnd_hsync;
  wire          bgnd_hblnk;  
  wire          bgnd_vsync;
  wire          bgnd_vblnk;

  timing timing_inst (
    .tc_hsblnk(tc_hsblnk), //input
    .tc_hssync(tc_hssync), //input
    .tc_hesync(tc_hesync), //input
    .tc_heblnk(tc_heblnk), //input
    .hcount(bgnd_hcount), //output
    .hsync(VGA_HSYNC_INT), //output
    .hblnk(bgnd_hblnk), //output
    .tc_vsblnk(tc_vsblnk), //input
    .tc_vssync(tc_vssync), //input
    .tc_vesync(tc_vesync), //input
    .tc_veblnk(tc_veblnk), //input
    .vcount(bgnd_vcount), //output
    .vsync(VGA_VSYNC_INT), //output
    .vblnk(bgnd_vblnk), //output
    .restart(reset),
    .clk(pclk));

  /////////////////////////////////////////
  // V/H SYNC and DE generator
  /////////////////////////////////////////
  assign active = !bgnd_hblnk && !bgnd_vblnk;


  always @ (posedge pclk)
  begin
    hsync <= VGA_HSYNC_INT ^ hvsync_polarity ;
    vsync <= VGA_VSYNC_INT ^ hvsync_polarity ;
    VGA_HSYNC <= hsync;
    VGA_VSYNC <= vsync;

    active_q <= active;
    de <= active_q;
  end



endmodule

